@@ -1,5 +1,5 @@
 #  File src/library/base/R/factor.R
-#  Part of the R package, http://www.R-project.org
+#  Part of the R package, https://www.R-project.org
 #
 #  Copyright (C) 1995-2015 The R Core Team
 #
@@ -14,7 +14,7 @@
 #  GNU General Public License for more details.
 #
 #  A copy of the GNU General Public License is available at
-#  http://www.r-project.org/Licenses/
+#  https://www.R-project.org/Licenses/
 
 factor <- function(x = character(), levels, labels = levels,
                    exclude = NA, ordered = is.ordered(x), nmax = NA)
@@ -108,7 +108,7 @@ as.vector.factor <- function(x, mode="any")
 	as.vector(unclass(x), mode)
 }
 
-as.character.factor <- function(x,...) levels(x)[x]
+as.character.factor <- function(x,...) .Internal(asCharacterFactor(x))
 
 as.logical.factor <- function(x,...) as.logical(levels(x))[x]
 
