@@ -1,7 +1,7 @@
 #  File src/library/base/R/library.R
-#  Part of the R package, http://www.R-project.org
+#  Part of the R package, https://www.R-project.org
 #
-#  Copyright (C) 1995-2015 The R Core Team
+#  Copyright (C) 1995-2016 The R Core Team
 #
 #  This program is free software; you can redistribute it and/or modify
 #  it under the terms of the GNU General Public License as published by
@@ -14,7 +14,7 @@
 #  GNU General Public License for more details.
 #
 #  A copy of the GNU General Public License is available at
-#  http://www.r-project.org/Licenses/
+#  https://www.R-project.org/Licenses/
 
 testPlatformEquivalence <-
 function(built, run)
@@ -24,7 +24,7 @@ function(built, run)
     built <- gsub("([^-]*)-([^-]*)-(.*)", "\\1-\\3", built)
     run <- gsub("([^-]*)-([^-]*)-(.*)", "\\1-\\3", run)
     ## Mac OS X supports multiple CPUs by using 'universal' binaries
-    if (grepl("^universal-darwin", built) && nzchar(.Platform$r_arch))
+    if (startsWith(built, "universal-darwin") && nzchar(.Platform$r_arch))
         built <- sub("^universal", R.version$arch, built)
     ## allow for small mismatches, e.g. OS version number and i686 vs i586.
     length(agrep(built, run)) > 0
@@ -82,7 +82,7 @@ function(package, help, pos = 2, lib.loc = NULL, character.only = FALSE,
             ## allow mismatches if r_arch is in use, e.g.
             ## i386-gnu-linux vs x86-gnu-linux depending on
             ## build system.
-            if(!nzchar(r_arch) && length(grep("\\w", platform)) &&
+            if(!nzchar(r_arch) && grepl("\\w", platform) &&
                !testPlatformEquivalence(platform, R.version$platform))
                 stop(gettextf("package %s was built for %s",
                               sQuote(pkgname), platform),
@@ -118,11 +118,15 @@ function(package, help, pos = 2, lib.loc = NULL, character.only = FALSE,
                 if(pkg %in% agreed) return()
             } else agreed <- character()
             if(!interactive())
-                stop(gettextf("package %s has a license that you need to accept in an interactive session", sQuote(pkg)), domain = NA)
+                stop(gettextf(
+                    "package %s has a license that you need to accept in an interactive session",
+                              sQuote(pkg)), domain = NA)
             lfiles <- file.path(pkgpath, c("LICENSE", "LICENCE"))
             lfiles <- lfiles[file.exists(lfiles)]
             if(length(lfiles)) {
-                message(gettextf("package %s has a license that you need to accept after viewing", sQuote(pkg)), domain = NA)
+                message(gettextf(
+                    "package %s has a license that you need to accept after viewing",
+                                 sQuote(pkg)), domain = NA)
                 readline("press RETURN to view license")
                 encoding <- pkgInfo$DESCRIPTION["Encoding"]
                 if(is.na(encoding)) encoding <- ""
@@ -130,11 +134,14 @@ function(package, help, pos = 2, lib.loc = NULL, character.only = FALSE,
                 if(encoding == "latin1") encoding <- "cp1252"
                 file.show(lfiles[1L], encoding = encoding)
             } else {
-                message(gettextf("package %s has a license that you need to accept:\naccording to the DESCRIPTION file it is", sQuote(pkg)), domain = NA)
-                message(pkgInfo$DESCRIPTION["License"], domain = NA)
+                message(gettextf(paste("package %s has a license that you need to accept:",
+				       "according to the DESCRIPTION file it is",
+				       "%s", sep="\n"),
+				 sQuote(pkg),
+				 pkgInfo$DESCRIPTION["License"]), domain = NA)
             }
-            choice <- menu(c("accept", "decline"),
-                           title = paste("License for", sQuote(pkg)))
+            choice <- utils::menu(c("accept", "decline"),
+                                  title = paste("License for", sQuote(pkg)))
             if(choice != 1)
                 stop(gettextf("license for package %s not accepted",
                               sQuote(package)), domain = NA, call. = FALSE)
@@ -153,7 +160,7 @@ function(package, help, pos = 2, lib.loc = NULL, character.only = FALSE,
         else {
             ## A package will have created a generic
             ## only if it has created a formal method.
-            length(objects(env, pattern="^\\.__T", all.names=TRUE)) == 0L
+            length(grep(pattern="^\\.__T", names(env))) == 0L
         }
     }
 
@@ -165,15 +172,15 @@ function(package, help, pos = 2, lib.loc = NULL, character.only = FALSE,
                        ".packageName", ".noGenerics", ".required",
                        ".no_S3_generics", ".Depends", ".requireCachedGenerics")
         sp <- search()
-        lib.pos <- match(pkgname, sp)
+        lib.pos <- which(sp == pkgname)
         ## ignore generics not defined for the package
-        ob <- objects(lib.pos, all.names = TRUE)
+        ob <- names(as.environment(lib.pos))
         if(!nogenerics) {
             ##  Exclude generics that are consistent with implicit generic
             ## from another package.  A better test would be to move this
             ## down into the loop and test against specific other package name
             ## but subtle conflicts like that are likely to be found elsewhere
-	    these <- ob[substr(ob, 1L, 6L) == ".__T__"]
+	    these <- ob[startsWith(ob,".__T__")]
             gen  <- gsub(".__T__(.*):([^:]+)", "\\1", these)
             from <- gsub(".__T__(.*):([^:]+)", "\\2", these)
             gen <- gen[from != package]
@@ -183,11 +190,11 @@ function(package, help, pos = 2, lib.loc = NULL, character.only = FALSE,
 	ipos <- seq_along(sp)[-c(lib.pos,
 				 match(c("Autoloads", "CheckExEnv"), sp, 0L))]
         for (i in ipos) {
-            obj.same <- match(objects(i, all.names = TRUE), ob, nomatch = 0L)
+            obj.same <- match(names(as.environment(i)), ob, nomatch = 0L)
             if (any(obj.same > 0)) {
                 same <- ob[obj.same]
                 same <- same[!(same %in% dont.mind)]
-                Classobjs <- grep("^\\.__", same)
+                Classobjs <- which(startsWith(same,".__"))
                 if(length(Classobjs)) same <- same[-Classobjs]
                 ## report only objects which are both functions or
                 ## both non-functions.
@@ -213,7 +220,8 @@ function(package, help, pos = 2, lib.loc = NULL, character.only = FALSE,
                                                        sQuote(package)),
                                               domain = NA)
                     }
-		    msg <- .maskedMsg(same, pkg = sQuote(sp[i]), by = i < lib.pos)
+		    msg <- .maskedMsg(sort(same), pkg = sQuote(sp[i]),
+                                      by = i < lib.pos)
 		    packageStartupMessage(msg, domain = NA)
                 }
             }
@@ -243,7 +251,10 @@ function(package, help, pos = 2, lib.loc = NULL, character.only = FALSE,
             ## The methods package caches all other pkgs when it is
             ## attached.
 
-            pkgpath <- find.package(package, lib.loc, quiet = TRUE,
+            ## Too extreme (unfortunately; warning too often):
+	    ## pkgpath <- find.package(package, lib.loc, quiet = TRUE, verbose = !quietly)
+	    ##   'verbose' here means to warn about packages found more than once
+	    pkgpath <- find.package(package, lib.loc, quiet = TRUE,
                                     verbose = verbose)
             if(length(pkgpath) == 0L) {
                 txt <- if(length(lib.loc))
@@ -275,7 +286,8 @@ function(package, help, pos = 2, lib.loc = NULL, character.only = FALSE,
             if(is.character(pos)) {
                 npos <- match(pos, search())
                 if(is.na(npos)) {
-                    warning(gettextf("%s not found on search path, using pos = 2", sQuote(pos)), domain = NA)
+                    warning(gettextf("%s not found on search path, using pos = 2",
+                                     sQuote(pos)), domain = NA)
                     pos <- 2
                 } else pos <- npos
             }
@@ -287,20 +299,23 @@ function(package, help, pos = 2, lib.loc = NULL, character.only = FALSE,
             ## takes over.
             if (packageHasNamespace(package, which.lib.loc)) {
 		if (isNamespaceLoaded(package)) {
-                    # Already loaded.  Does the version match?
+                    ## Already loaded.  Does the version match?
                     newversion <- as.numeric_version(pkgInfo$DESCRIPTION["Version"])
                     oldversion <- as.numeric_version(getNamespaceVersion(package))
                     if (newversion != oldversion) {
-                    	# No, so try to unload the previous one
+                    	## No, so try to unload the previous one
                     	res <- try(unloadNamespace(package))
                     	if (inherits(res, "try-error"))
-                    	    stop(gettextf("Package %s version %s cannot be unloaded", sQuote(package), oldversion, domain = "R-base"))
+                    	    stop(gettextf("Package %s version %s cannot be unloaded",
+					  sQuote(package), oldversion), domain=NA)
                     }
                 }
                 tt <- try({
-                    ns <- loadNamespace(package, c(which.lib.loc, lib.loc))
+                    attr(package, "LibPath") <- which.lib.loc
+                    ns <- loadNamespace(package, lib.loc)
                     env <- attachNamespace(ns, pos = pos, deps)
                 })
+                attr(package, "LibPath") <- NULL
                 if (inherits(tt, "try-error"))
                     if (logical.return)
                         return(FALSE)
@@ -436,7 +451,7 @@ function(package, help, pos = 2, lib.loc = NULL, character.only = FALSE,
     if (logical.return)
 	TRUE
     else invisible(.packages())
-}
+} ## {library}
 
 format.libraryIQR <-
 function(x, ...)
@@ -626,7 +641,7 @@ function(all.available = FALSE, lib.loc = NULL)
         return(unique(ans))
     } ## else
     s <- search()
-    return(invisible(substring(s[substr(s, 1L, 8L) == "package:"], 9)))
+    invisible(.rmpkg(s[substr(s, 1L, 8L) == "package:"]))
 }
 
 path.package <-
@@ -675,14 +690,10 @@ function(package = NULL, lib.loc = NULL, quiet = FALSE,
                       "splines", "stats4", "tcltk"))
         return(file.path(.Library, package))
 
-    use_loaded <- FALSE
     if(is.null(package)) package <- .packages()
-    if(is.null(lib.loc)) {
-        use_loaded <- TRUE
-        lib.loc <- .libPaths()
-    }
-
     if(!length(package)) return(character())
+    if(use_loaded <- is.null(lib.loc))
+	lib.loc <- .libPaths()
 
     bad <- character()
     out <- character()
@@ -738,7 +749,7 @@ function(package = NULL, lib.loc = NULL, quiet = FALSE,
         if(length(paths) > 1L) {
             ## If a package was found more than once ...
             paths <- paths[1L]
-            if(verbose)
+	    if(verbose)
                 warning(gettextf("package %s found more than once,\nusing the one found in %s",
                                  sQuote(pkg), sQuote(paths)), domain = NA)
         }
@@ -819,14 +830,7 @@ function(pkgInfo, quietly = FALSE, lib.loc = NULL, useImports = FALSE)
                              package = pkg, lib.loc = lib.loc)
         if (nzchar(pfile))
             as.numeric_version(readRDS(pfile)$DESCRIPTION["Version"])
-        else
-            NULL
-    }
-    .findAllVersions <- function(pkg, lib.loc = NULL) {
-        if (is.null(lib.loc))
-            lib.loc <- .libPaths()
-        do.call(c, Filter(Negate(is.null),
-                          lapply(lib.loc, .findVersion, pkg=pkg)))
+        ## else NULL
     }
     pkgs <- unique(names(pkgInfo$Depends))
     pkgname <- pkgInfo$DESCRIPTION["Package"]
@@ -846,20 +850,21 @@ function(pkgInfo, quietly = FALSE, lib.loc = NULL, useImports = FALSE)
             if (!sufficient) {
                 if (is.null(lib.loc))
                     lib.loc <- .libPaths()
-                versions <- .findAllVersions(pkg, lib.loc)
+		allV <- lapply(lib.loc, .findVersion, pkg=pkg)
+		versions <- do.call(c, allV[iV <- which(!vapply(allV, is.null, NA))])
                 sufficient <- vapply(versions, dep$op, logical(1L), target)
                 if (any(sufficient)) {
                     warning(gettextf("version %s of %s masked by %s in %s",
                                      versions[which(sufficient)[1L]],
                                      sQuote(pkg),
                                      current,
-                                     lib.loc[which(sufficient)[1L]-1L]),
+				     lib.loc[iV[!sufficient][1L]]),
                             call. = FALSE, domain = NA)
                 }
-                if (attached)
-                    msg <- "package %s %s is loaded, but %s %s is required by %s"
-                else
-                    msg <- "package %s %s was found, but %s %s is required by %s"
+		msg <- if (attached)
+			   "package %s %s is loaded, but %s %s is required by %s"
+		       else
+			   "package %s %s was found, but %s %s is required by %s"
                 stop(gettextf(msg, sQuote(pkg), current, dep$op,
                               target, sQuote(pkgname)),
                      call. = FALSE, domain = NA)
