@@ -1,7 +1,7 @@
 #  File src/library/base/R/seq.R
-#  Part of the R package, http://www.R-project.org
+#  Part of the R package, https://www.R-project.org
 #
-#  Copyright (C) 1995-2012 The R Core Team
+#  Copyright (C) 1995-2015 The R Core Team
 #
 #  This program is free software; you can redistribute it and/or modify
 #  it under the terms of the GNU General Public License as published by
@@ -14,7 +14,7 @@
 #  GNU General Public License for more details.
 #
 #  A copy of the GNU General Public License is available at
-#  http://www.r-project.org/Licenses/
+#  https://www.R-project.org/Licenses/
 
 seq <- function(...) UseMethod("seq")
 
@@ -24,8 +24,10 @@ seq.default <-
 {
     if((One <- nargs() == 1L) && !missing(from)) {
 	lf <- length(from)
-	return(if(mode(from) == "numeric" && lf == 1L) 1L:from else
-	       if(lf) 1L:lf else integer())
+	return(if(mode(from) == "numeric" && lf == 1L) {
+            if(!is.finite(from)) stop("'from' cannot be NA, NaN or infinite")
+            1L:from
+        } else if(lf) 1L:lf else integer())
     }
     if(!missing(along.with)) {
 	length.out <- length(along.with)
@@ -40,14 +42,13 @@ seq.default <-
         }
 	length.out <- ceiling(length.out)
     }
-    if(!missing(...))
-        warning(sprintf(ngettext(length(list(...)),
-                                 "extra argument %s will be disregarded",
-                                 "extra arguments %s will be disregarded"),
-			 paste(sQuote(names(list(...))), collapse = ", ")),
-		domain = NA)
+    chkDots(...)
     if (!missing(from) && length(from) != 1L) stop("'from' must be of length 1")
     if (!missing(to) && length(to) != 1L) stop("'to' must be of length 1")
+    if (!missing(from) && !is.finite(from))
+        stop("'from' cannot be NA, NaN or infinite")
+    if (!missing(to) && !is.finite(to))
+        stop("'to' cannot be NA, NaN or infinite")
     if(is.null(length.out))
 	if(missing(by))
 	    from:to
