@@ -1,5 +1,5 @@
 #  File src/library/stats/R/family.R
-#  Part of the R package, http://www.R-project.org
+#  Part of the R package, https://www.R-project.org
 #
 #  Copyright (C) 1995-2012 The R Core Team
 #
@@ -14,7 +14,7 @@
 #  GNU General Public License for more details.
 #
 #  A copy of the GNU General Public License is available at
-#  http://www.r-project.org/Licenses/
+#  https://www.R-project.org/Licenses/
 
 family <- function(object, ...) UseMethod("family")
 
@@ -532,9 +532,8 @@ inverse.gaussian <- function(link = "1/mu^2")
     })
     validmu <- function(mu) TRUE
     simfun <- function(object, nsim) {
-        if(is.null(tryCatch(loadNamespace("SuppDists"),
-                            error = function(e) NULL)))
-            stop("need CRAN package 'SuppDists' for the 'inverse.gaussian' family")
+        if(!requireNamespace("SuppDists", quietly = TRUE))
+            stop("need CRAN package 'SuppDists' for simulation from the 'inverse.gaussian' family")
         wts <- object$prior.weights
         if (any(wts != 1)) message("using weights as inverse variances")
         ftd <- fitted(object)
