@@ -4,11 +4,10 @@
  * According to the GAMFIT sources, this was derived from code by
  * Finbarr O'Sullivan.
  */
-#include <R.h>
-#include <Rmath.h>
-#include <R_ext/PrtUtil.h>
 
 #include "modreg.h"
+#include <math.h>
+#include <Rmath.h>
 
 /* sbart() : The cubic spline smoother
    -------
@@ -61,6 +60,7 @@ void F77_SUB(sbart)
    lspar, uspar lower and upper values for spar search;  0.,1. are good values
    tol, eps	used in Golden Search routine
    isetup	setup indicator [initially 0
+   NB: this alters that, and it is a constant in the caller!
    icrit	indicator saying which cross validation score is to be computed
 		0: none ;  1: GCV ;  2: CV ;  3: 'df matching'
    ld4		the leading dimension of abd (ie ld4=4)
