@@ -1,7 +1,7 @@
 #  File src/library/tools/R/dynamicHelp.R
-#  Part of the R package, http://www.R-project.org
+#  Part of the R package, https://www.R-project.org
 #
-#  Copyright (C) 1995-2015 The R Core Team
+#  Copyright (C) 1995-2016 The R Core Team
 #
 #  This program is free software; you can redistribute it and/or modify
 #  it under the terms of the GNU General Public License as published by
@@ -14,7 +14,7 @@
 #  GNU General Public License for more details.
 #
 #  A copy of the GNU General Public License is available at
-#  http://www.r-project.org/Licenses/
+#  https://www.R-project.org/Licenses/
 
 
 ## This may be asked for
@@ -64,7 +64,7 @@ httpd <- function(path, query, ...)
     {
     	bool <- function(x) as.logical(as.numeric(x))
         res <- if(identical(names(query), "category")) {
-            help.search(keyword = query, verbose = 1L, use_UTF8 = TRUE)
+            utils::help.search(keyword = query, verbose = 1L, use_UTF8 = TRUE)
         } else if(identical(names(query), "results")) {
             utils:::.hsearch_results()
         } else {
@@ -107,7 +107,7 @@ httpd <- function(path, query, ...)
             args$fields <- fields
             args$use_UTF8 <- TRUE
             args$types <- types
-            do.call(help.search, args)
+            do.call(utils::help.search, args)
         }
         types <- res$types
         res <- res$matches
@@ -166,13 +166,13 @@ httpd <- function(path, query, ...)
         concepts <- utils::hsearch_db_concepts()
         s <- concepts$Concept
         out <-
-            c(tools:::HTMLheader("Help search concepts"),
+            c(HTMLheader("Help search concepts"),
               c("",
                 "<table>",
                 "<tr><th style=\"text-align: left\">Concept</th><th>Frequency</th><th>Packages</th><tr>",
                 paste0("<tr><td>",
                        "<a href=\"/doc/html/Search?pattern=",
-                       vapply(reQuote(s), URLencode, "", reserved = TRUE),
+                       vapply(reQuote(s), utils::URLencode, "", reserved = TRUE),
                        "&fields.concept=1&agrep=0\">",
                        shtmlify(substring(s, 1, 80)),
                        "</a>",
@@ -190,7 +190,7 @@ httpd <- function(path, query, ...)
     .HTML_hsearch_db_keywords <- function() {
         keywords <- utils::hsearch_db_keywords()
         out <-
-            c(tools:::HTMLheader("Help search keywords"),
+            c(HTMLheader("Help search keywords"),
               c("",
                 "<table>",
                 "<tr><th style=\"text-align: left\">Keyword</th><th style=\"text-align: left\">Concept</th><th>Frequency</th><th>Packages</th><tr>",
@@ -309,9 +309,9 @@ httpd <- function(path, query, ...)
     	topic <- sub(topicRegexp, "\\2", path)
         ## if a package is specified, look there first, then everywhere
     	if (!is.null(pkg)) # () avoids deparse here
-    	    file <- help(topic, package = (pkg), help_type = "text")
+    	    file <- utils::help(topic, package = (pkg), help_type = "text")
     	if (!length(file))
-            file <- help(topic, help_type = "text", try.all.packages = TRUE)
+            file <- utils::help(topic, help_type = "text", try.all.packages = TRUE)
 	if (!length(file)) {
             msg <- gettextf("No help found for topic %s in any package.",
                             mono(topic))
@@ -402,8 +402,8 @@ httpd <- function(path, query, ...)
             if(is.na(tmp)) {
                 msg <- gettextf("Link %s in package %s could not be located",
                                 mono(helpdoc), mono(pkg))
-                files <- help(helpdoc, help_type = "text",
-                              try.all.packages = TRUE)
+                files <- utils::help(helpdoc, help_type = "text",
+                                     try.all.packages = TRUE)
                 if (length(files)) {
                     path <- dirname(dirname(files))
                     files <- paste0('/library/', basename(path), '/html/',
@@ -488,7 +488,16 @@ httpd <- function(path, query, ...)
 				")' in the console.")) )
     } else if (grepl(newsRegexp, path)) {
     	pkg <- sub(newsRegexp, "\\1", path)
-    	formatted <- toHTML(news(package = pkg),
+    	if (!is.null(query) && !is.na(subset <- query["subset"])) {
+    	    # See utils:::print.news_db for the encoding of the subset
+    	    rle <- strsplit(subset, "_")[[1]]
+    	    rle <- structure(list(lengths = as.numeric(rle),
+    	    	                  values = rep(c(TRUE, FALSE), length.out = length(rle))),
+    	    	             class = "rle")
+    	    news <- news(inverse.rle(rle)[-1], package = pkg)
+	} else
+    	    news <- news(package = pkg)
+    	formatted <- toHTML(news,
     		            title=paste("NEWS in package", sQuote(pkg)),
     			    up="html/00Index.html")
         if (length(formatted))
@@ -509,7 +518,7 @@ httpd <- function(path, query, ...)
     	pkg <- sub(cssRegexp, "\\1", path)
         return( list(file = system.file("html", "R.css", package = pkg),
                      "content-type" = "text/css") )
-    } else if (grepl("^/library/", path)) {
+    } else if (startsWith(path, "/library/")) {
         descRegexp <- "^/library/+([^/]+)/+DESCRIPTION$"
         if(grepl(descRegexp, path)) {
             pkg <- sub(descRegexp, "\\1", path)
@@ -535,7 +544,7 @@ httpd <- function(path, query, ...)
         if(file.exists(file))
             list(file = file, "content-type" = mime_type(path))
         else {
-            url <- "http://cran.r-project.org/bin/windows/base/rw-FAQ.html"
+            url <- "https://cran.r-project.org/bin/windows/base/rw-FAQ.html"
 	    return(list(payload = paste0('Redirect to <a href="', url, '">"',
                                          url, '"</a>'),
 	    		"content-type" = 'text/html',
@@ -554,10 +563,10 @@ httpd <- function(path, query, ...)
             ## tarball has pre-built version of R-admin.html
             list(file = file, "content-type" = mime_type(path))
         } else {
-            ## url <- "http://cran.r-project.org/manuals.html"
+            ## url <- "https://cran.r-project.org/manuals.html"
             version <-
                 if(grepl("unstable", R.version$status)) "r-devel" else "r-patched"
-            url <- file.path("http://cran.r-project.org/doc/manuals",
+            url <- file.path("https://cran.r-project.org/doc/manuals",
                              version, basename(path))
 	    return(list(payload = paste0('Redirect to <a href="', url, '">"',
                                          url, '"</a>'),
@@ -566,7 +575,7 @@ httpd <- function(path, query, ...)
 	    		"status code" = 302L)) # temporary redirect
         }
     } else {
-        if(grepl("^/doc/", path)) {
+        if(startsWith(path, "/doc/")) {
             ## /doc/AUTHORS and so on.
             file <- file.path(R.home("doc"), sub("^/doc", "", path))
         } else return(error_page(gettextf("unsupported URL %s", mono(path))))
@@ -609,7 +618,6 @@ startDynamicHelp <- function(start = TRUE)
     if(!start && (port <= 0L))
         stop("no running server to stop")
     if (start) {
-        message("starting httpd help server ...", appendLF = FALSE)
         utils::flush.console()
         OK <- FALSE
         ports <- getOption("help.ports")
@@ -622,6 +630,9 @@ startDynamicHelp <- function(start = TRUE)
             ports <- 10000 + 22000*((stats::runif(10) + unclass(Sys.time())/300) %% 1)
         }
         ports <- as.integer(ports)
+	if (all(ports == 0))
+	    return(invisible(0))
+        message("starting httpd help server ...", appendLF = FALSE)
         for(i in seq_along(ports)) {
             ## the next can throw an R-level error,
             ## so do not assign port unless it succeeds.
