@@ -1,7 +1,7 @@
 #  File src/library/utils/R/sessionInfo.R
-#  Part of the R package, http://www.R-project.org
+#  Part of the R package, https://www.R-project.org
 #
-#  Copyright (C) 1995-2014 The R Core Team
+#  Copyright (C) 1995-2016 The R Core Team
 #
 #  This program is free software; you can redistribute it and/or modify
 #  it under the terms of the GNU General Public License as published by
@@ -14,7 +14,7 @@
 #  GNU General Public License for more details.
 #
 #  A copy of the GNU General Public License is available at
-#  http://www.r-project.org/Licenses/
+#  https://www.R-project.org/Licenses/
 
 sessionInfo <- function(package = NULL)
 {
@@ -36,10 +36,10 @@ sessionInfo <- function(package = NULL)
                    "Linux" = if(file.exists("/etc/os-release")) {
     ## http://www.freedesktop.org/software/systemd/man/os-release.html
                        tmp <- readLines("/etc/os-release")
-                       t2 <- if (any(grepl("^PRETTY_NAME=", tmp)))
+                       t2 <- if (any(startsWith(tmp, "PRETTY_NAME=")))
                            sub("^PRETTY_NAME=", "",
                                grep("^PRETTY_NAME=", tmp, value = TRUE)[1L])
-                       else if (any(grepl("^NAME", tmp)))
+                       else if (any(startsWith(tmp, "NAME")))
                            ## could check for VERSION or VERSION_ID
                            sub("^NAME=", "",
                                grep("^NAME=", tmp, value = TRUE)[1L])
@@ -79,8 +79,9 @@ sessionInfo <- function(package = NULL)
     if(is.null(package)){
         package <- grep("^package:", search(), value=TRUE)
         # weed out environments which are not really packages
-        keep <- sapply(package, function(x) x == "package:base" || !is.null(attr(as.environment(x), "path")))
-        package <- sub("^package:", "", package[keep])
+        keep <- vapply(package, function(x) x == "package:base"
+                       || !is.null(attr(as.environment(x), "path")), NA)
+        package <- .rmpkg(package[keep])
     }
 
     ## no need to re-encode given what we extract.
