@@ -1,7 +1,7 @@
 #  File src/library/utils/R/tar.R
 #  Part of the R package, https://www.R-project.org
 #
-#  Copyright (C) 1995-2014 The R Core Team
+#  Copyright (C) 1995-2016 The R Core Team
 #
 #  This program is free software; you can redistribute it and/or modify
 #  it under the terms of the GNU General Public License as published by
@@ -245,7 +245,7 @@ untar2 <- function(tarfile, files = NULL, list = FALSE, exdir = ".",
                             warn1 <- c(warn1, "restoring symbolic link as a file copy")
                    } else {
                        mydir.create(dirname(name))
-                       od <- setwd(dirname(name))
+                       od0 <- setwd(dirname(name))
                        nm <- basename(name)
                        unlink(nm)
                        if(!file.symlink(name2, nm)) { # will give a warning
@@ -255,7 +255,7 @@ untar2 <- function(tarfile, files = NULL, list = FALSE, exdir = ".",
                            else
                                warning(gettextf("failed to copy %s to %s", sQuote(from), sQuote(name)), domain = NA)
                        }
-                       setwd(od)
+                       setwd(od0)
                    }
                 }
             }
@@ -329,6 +329,14 @@ tar <- function(tarfile, files = NULL,
                 compression_level = 6, tar = Sys.getenv("tar"),
                 extra_flags = "")
 {
+    files <- ## list the files before 'tarfile' is created!
+	if(is.null(files)) ## is fine
+	    list.files(recursive = TRUE, all.files = TRUE,
+		       full.names = TRUE, include.dirs = TRUE)
+	else ## is *not* ok when 'files' are simple files !
+	    ## list.files(path, ....) : first argument are *directories*
+	    list.files(files, recursive = TRUE, all.files = TRUE,
+		       full.names = TRUE, include.dirs = TRUE)
     if(is.character(tarfile)) {
         if(nzchar(tar) && tar != "internal") {
             ## FIXME: could pipe through gzip etc: might be safer for xz
@@ -383,9 +391,6 @@ tar <- function(tarfile, files = NULL,
     }
     warn1 <- character()
 
-    files <- list.files(files, recursive = TRUE, all.files = TRUE,
-                        full.names = TRUE, include.dirs = TRUE)
-
     invalid_uid <- invalid_gid <- FALSE
     for (f in unique(files)) {
         info <- file.info(f)
