@@ -1,6 +1,6 @@
 /*
  *  R : A Computer Language for Statistical Data Analysis
- *  Copyright (C)  2001-5   The R Core Team.
+ *  Copyright (C)  2001-2015   The R Core Team.
  *
  *  This program is free software; you can redistribute it and/or modify
  *  it under the terms of the GNU General Public License as published by
@@ -14,7 +14,7 @@
  *
  *  You should have received a copy of the GNU General Public License
  *  along with this program; if not, a copy is available at
- *  http://www.r-project.org/Licenses/
+ *  https://www.R-project.org/Licenses/
  */
 
 /* <UTF8> chars are only handled as a whole */
@@ -23,8 +23,6 @@
 #include <config.h>
 #endif
 
-#ifdef HAVE_SOCKETS
-
 
 /* ------------------- socket connections  --------------------- */
 
@@ -88,7 +86,7 @@ static Rboolean sock_open(Rconnection con)
     }
     this->fd = sock;
 
-    mlen = strlen(con->mode);
+    mlen = (int) strlen(con->mode);
     con->isopen = TRUE;
     if(mlen >= 2 && con->mode[mlen - 1] == 'b') con->text = FALSE;
     else con->text = TRUE;
@@ -104,11 +102,11 @@ static void sock_close(Rconnection con)
     con->isopen = FALSE;
 }
 
-static int sock_read_helper(Rconnection con, void *ptr, size_t size)
+static ssize_t sock_read_helper(Rconnection con, void *ptr, size_t size)
 {
     Rsockconn this = (Rsockconn)con->private;
-    int res;
-    int nread = 0, n;
+    ssize_t res;
+    size_t nread = 0, n;
 
     con->incomplete = FALSE;
     do {
@@ -148,7 +146,7 @@ static int sock_read_helper(Rconnection con, void *ptr, size_t size)
 static int sock_fgetc_internal(Rconnection con)
 {
     unsigned char c;
-    int n;
+    ssize_t n;
 
     n = sock_read_helper(con, (char *)&c, 1);
     return (n == 1) ? c : R_EOF;
@@ -157,15 +155,17 @@ static int sock_fgetc_internal(Rconnection con)
 static size_t sock_read(void *ptr, size_t size, size_t nitems,
 			Rconnection con)
 {
-    return sock_read_helper(con, ptr, size * nitems)/size;
+    ssize_t n = sock_read_helper(con, ptr, size * nitems)/size;
+    return n > 0 ? n : 0;
 }
 
 static size_t sock_write(const void *ptr, size_t size, size_t nitems,
 			 Rconnection con)
 {
     Rsockconn this = (Rsockconn)con->private;
-
-    return R_SockWrite(this->fd, ptr, size * nitems, this->timeout)/size;
+    ssize_t n = R_SockWrite(this->fd, ptr, (int)(size * nitems),
+			    this->timeout)/size;
+    return n > 0 ? n : 0;
 }
 
 Rconnection in_R_newsock(const char *host, int port, int server,
@@ -204,5 +204,3 @@ Rconnection in_R_newsock(const char *host, int port, int server,
     ((Rsockconn)new->private)-> timeout = timeout;
     return new;
 }
-
-#endif /* HAVE_SOCKETS */
