@@ -11,7 +11,7 @@ str(ordered(ff, exclude=NULL))
 if(require(survival)) {
     (sa <- Surv(aml$time, aml$status))
     str(sa)
-    detach("package:survival")
+    detach("package:survival", unload = TRUE)
 }
 ## were different, the last one failed in 1.6.2 (at least)
 
@@ -36,8 +36,8 @@ if(require(MASS)) {
 
 ## PR#7829 model.tables & replications
 if(require(MASS)) {
-oats.aov <- aov(Y ~ B + V + N + V:N, data=oats[-1,])
-model.tables(oats.aov, "means", cterms=c("N", "V:N"))
+    oats.aov <- aov(Y ~ B + V + N + V:N, data=oats[-1,])
+    model.tables(oats.aov, "means", cterms=c("N", "V:N"))
 }
 ## wrong printed output in 2.1.0
 
@@ -86,16 +86,17 @@ print(1.001, digits=16)
 
 
 ## as.matrix.data.frame with coercion
-library(survival)
-soa <- Surv(1:5, c(0, 0, 1, 0, 1))
-df.soa <- data.frame(soa)
-as.matrix(df.soa) # numeric result
-df.soac <- data.frame(soa, letters[1:5])
-as.matrix(df.soac) # character result
+if(require("survival")) {
+    soa <- Surv(1:5, c(0, 0, 1, 0, 1))
+    df.soa <- data.frame(soa)
+    print(as.matrix(df.soa)) # numeric result
+    df.soac <- data.frame(soa, letters[1:5])
+    print(as.matrix(df.soac)) # character result
+    detach("package:survival", unload = TRUE)
+}
 ## failed in 2.8.1
 
 ## wish of PR#13505
-utils::data(npk, package="MASS")
 npk.aov <- aov(yield ~ block + N * P + K, npk)
 foo <- proj(npk.aov)
 cbind(npk, foo)
@@ -111,6 +112,7 @@ if(suppressMessages(require("Matrix"))) {
   x3 <- x2; x3[4:5] <- x2[5:4]
   print(xtabs(~ x1 + x2, sparse= TRUE, exclude = 'c'))
   print(xtabs(~ x1 + x3, sparse= TRUE, exclude = 'c'))
+  detach("package:Matrix")
   ## failed in R <= 2.13.1
 }
 
@@ -122,5 +124,75 @@ contr.sum(2) # needed drop=FALSE at one point.
 ## xtabs did not exclude levels from factors
 x1 <- c('a','b','a','b','c', NA)
 x2 <- factor(x1, exclude=NULL)
-print(xtabs(~ x1 + x2, na.action = na.pass)) 
-print(xtabs(~ x1 + x2, exclude = 'c', na.action = na.pass)) 
+print(xtabs(~ x1 + x2, na.action = na.pass))
+print(xtabs(~ x1 + x2, exclude = 'c', na.action = na.pass))
+
+
+## median should work by default for a suitable S4 class.
+## adapted from adaptsmoFMRI
+if(suppressMessages(require("Matrix"))) {
+    x <- matrix(c(1,2,3,4))
+    print(median(x))
+    print(median(as(x, "dgeMatrix")))
+    detach("package:Matrix")
+}
+
+## Various arguments were not duplicated:  PR#15352 to 15354
+x <- 5
+y <- 2
+f <- function (y) x
+numericDeriv(f(y),"y")
+x
+
+a<-list(1,2)
+b<-rep.int(a,c(2,2))
+b[[1]][1]<-9
+a[[1]]
+
+a <- numeric(1)
+x <- mget("a",as.environment(1))
+x
+a[1] <- 9
+x
+
+
+## needs MASS installed
+## PR#2586 labelling in alias()
+if(require("MASS")) {
+    Y <- c(0,1,2)
+    X1 <- c(0,1,0)
+    X2 <- c(0,1,0)
+    X3 <- c(0,0,1)
+    print(res <- alias(lm(Y ~ X1 + X2 + X3)))
+    stopifnot(identical(rownames(res[[2]]), "X2"))
+}
+## the error was in lm.(w)fit
+
+if(require("Matrix")) {
+ m1 <- m2 <- m <- matrix(1:12, 3,4)
+ dimnames(m2) <- list(LETTERS[1:3],
+                      letters[1:4])
+ dimnames(m1) <- list(NULL,letters[1:4])
+ M  <- Matrix(m)
+ M1 <- Matrix(m1)
+ M2 <- Matrix(m2)
+ ## Now, with a new ideal cbind(), rbind():
+ print(cbind(M, M1))
+ stopifnot(identical(cbind (M, M1),
+                     cbind2(M, M1)))
+ rm(M,M1,M2)
+ detach("package:Matrix", unload=TRUE)
+}##{Matrix}
+
+## Invalid UTF-8 strings
+x <- c("Jetz", "no", "chli", "z\xc3\xbcrit\xc3\xbc\xc3\xbctsch:",
+       "(noch", "ein", "bi\xc3\x9fchen", "Z\xc3\xbc", "deutsch)",
+       "\xfa\xb4\xbf\xbf\x9f")
+lapply(x, utf8ToInt)
+Encoding(x) <- "UTF-8"
+nchar(x, "b")
+try(nchar(x, "c"))
+try(nchar(x, "w"))
+nchar(x, "c", allowNA = TRUE)
+nchar(x, "w", allowNA = TRUE)
+## Results differed by platform, but some gave incorrect results on string 10.
